library(DBI)
library(RSQLite)
library(reshape2)
library(plyr)

sys.source(getOption("Kodi.R.path"), Kodi <- new.env())
for (name in Kodi$qw(
        char, groups, ss, ssq, qw, ordf, posix.ct
      ))
    assign(name, Kodi[[name]])

# ----------------------------------------------------------

dbs = list(
    sn.prefix = qw(d, w),
    h = c(
        dbConnect(dbDriver("SQLite"),
            dbname = "cheese-desserts.sqlite"),
        dbConnect(dbDriver("SQLite"),
            dbname = "cheese-cheesecake.sqlite")))

combine.tables = function(table.name)
    do.call(rbind, lapply(seq_along(dbs), function(i)
        transform(dbReadTable(dbs$h[[i]], table.name),
            sn = paste(dbs$sn.prefix[i], sn, sep = ""))))

subjects = ss(combine.tables("Subjects"), !is.na(completed_t))
row.names(subjects) = subjects$sn

dtable = ss(dcast(combine.tables("D"), sn ~ k, value.var = "v"))
row.names(dtable) = dtable$sn
dtable = dtable[row.names(subjects),]

timing = ordf(
    transform(
        ss(combine.tables("Timing"), char(sn) %in% row.names(subjects)),
        k = factor(k),
        d = received - first_sent),
    received, sn, first_sent)

images.table = read.delim("images.tsv", sep = "\t", header = T)
desserts = ss(images.table, category == "dessert")$file
cheesecake = ss(images.table, category == "cheesecake")$file

# ------------------------------------------------------------

sb = transform(subjects,
    task_version = factor(task_version, levels =
        unique(ordf(subjects, consented_t)$task_version)),
    tv = NA,
    first_seen_t = posix.ct(first_seen_t),
    consented_t = posix.ct(consented_t),
    completed_t = posix.ct(completed_t))
sb = transform(sb,
    tv = as.integer(task_version),
    choosing = NA,
    gender = dtable$gender,
    femphile = dtable$attracted_to_women,
    comments = dtable$comments,
    duration = round(`units<-`(completed_t - consented_t, "mins"), 2))
sb = transform(sb, choosing = factor(ifelse(tv == 1, "desserts", "women")))

perms = with(dtable, strsplit(
    ifelse(is.na(dessert_permutation), cheesecake_permutation, dessert_permutation),
    "|", fixed = T))
names(perms) = row.names(sb)

f = function(var)
   sapply(row.names(sb), function (sn)
       perms[[sn]][as.numeric(dtable[sn, var])])
sb = transform(sb,
    choice1 = f("choice1"),
    choice2 = f("choice2"),
    choice3 = f("choice3"))

# ------------------------------------------------------------

dsr = with(ss(sb, choosing == "desserts"), transform(
    data.frame(
        row.names = desserts,
        first = groups(factor(choice1, levels = desserts)),
        second = groups(factor(choice2, levels = desserts)),
        third = groups(factor(choice3, levels = desserts))),
    score = 3*first + 2*second + third,
    times.picked = first + second + third))

wmn = function(p)
    with(ssq(ss(sb, choosing == "women"), p), transform(
        data.frame(
            row.names = cheesecake,
            first = groups(factor(choice1, levels = cheesecake)),
            second = groups(factor(choice2, levels = cheesecake)),
            third = groups(factor(choice3, levels = cheesecake))),
        score = 3*first + 2*second + third,
        times.picked = first + second + third))

esa.analysis = function()
   {library(plyr)
    ss = subset

    print(dsr)
    print(length(unique(with(ss(sb, tv == 1),
        paste(choice1, choice2, choice3)))))
    x = ss(sb, tv == 1)[qw(choice1, choice2, choice3)]
    print(length(unique(apply(x, 1, function(r)
        paste(collapse = " ", sort(r))))))

    print(nrow(ss(sb, tv == 2)))
    print(nrow(ss(sb, tv == 2 & gender == "Male" & femphile == "Yes")))
    print(wmn(.(gender == "Male" & femphile == "Yes")))
    print(length(unique(with(
        ss(sb, tv == 2 & gender == "Male" & femphile == "Yes"),
        paste(choice1, choice2, choice3)))))
    x = ss(sb, tv == 2 & gender == "Male" & femphile == "Yes")[qw(choice1, choice2, choice3)]
    print(length(unique(apply(x, 1, function(r)
        paste(collapse = " ", sort(r))))))}

